module Asserter
  class Permission
    # Initialize a new permission.
    #
    # ==== Parameters
    # * <tt>permission</tt> - Permission
    #
    # ==== Returns
    # * <tt>Asserter::Permission</tt> - New instance.
    def initialize(permission)
      @permission_terms = permission.split('.')
    end

    # Validate a permission against an action.
    # Check each terms of a given permission against
    # each terms of a given action and compare them.
    #
    # If one of the permission's terms contains a
    # wildcard (*), we don't compare it and skip
    # to the next iteration.
    #
    # ==== Parameters
    # * <tt>action</tt> - Action to validate.
    #
    # ==== Returns
    # * <tt>Boolean</tt> - True if permission by pass action.
    def permit?(action)
      action_terms = action.split('.', @permission_terms.length)
      return false if action_terms.length != @permission_terms.length

      @permission_terms.zip(action_terms).each do |p_term, a_term|
        next if p_term == '*'
        return false if p_term != a_term
      end
    end
  end
end
